#!/bin/bash

# exit on errors
set -u -e

UPDATE_PARTITION="/dev/mmcblk0p10"
UPDATE_MNT="/update"
UPDATE_TMP=$UPDATE_MNT/gui-tmp
UPDATE_OK=$UPDATE_MNT/gui-ok
UPDATE_FINAL=$UPDATE_MNT/gui


function log()
{
	logger "$@"
	echo "$@" >/dev/console
}

function exit_trap() 
{
	rm -rf "$UPDATE_TMP"
	[ -e "$INFILE" ] && rm -f "$INFILE"
}

trap exit_trap ERR TERM INT

grep -q "$UPDATE_MNT" /proc/mounts || mount "$UPDATE_MNT"

case "$1" in
extract)
	INFILE="$2"
	# delete update directory
	[ -d "$UPDATE_TMP" ] && rm -rf "$UPDATE_TMP"
	[ -d "$UPDATE_OK" ] && rm -rf "$UPDATE_OK"
	[ -d "$UPDATE_FINAL" ] && rm -rf "$UPDATE_FINAL"

	if [ ! -f "$INFILE" ] ; then
		log "invalid filename $INFILE"
		exit 1
	fi

	mkdir -p "$UPDATE_TMP"
	if ! unzip "$INFILE" -d "$UPDATE_TMP" ; then
		log "extracting the image file failed"
		exit 1
	fi

	# to disable filesystem cache
	sync
	echo 1 > /proc/sys/vm/drop_caches
	echo 2 > /proc/sys/vm/drop_caches
	echo 3 > /proc/sys/vm/drop_caches

	# checking integrity of image

	OLDPATH=$PWD
	for path in "${UPDATE_TMP}/images" "${UPDATE_TMP}/sbin" "${UPDATE_TMP}/META-INF" ; do
		[ -d "$path" ] || log "cannot find path $path"
		cd "$path"
		if [ -f checksum2.sig ] ; then
			if ! gpg --ignore-time-conflict --homedir /tmp --primary-keyring /etc/gnupg/pubring.gpg --verify checksum2.sig ; then
				log "gpg signature2 for directory $path does not match"
				exit 1
			fi
			if ! sha512sum -c checksum2 ; then
				log "checksum for directory $path does not match"
				exit 1
			fi
		else
			if ! gpg --ignore-time-conflict --homedir /tmp --primary-keyring /etc/gnupg/pubring.gpg --verify checksum.sig ; then
				log "gpg signature for directory $path does not match"
				exit 1
			fi
			if ! md5sum -c checksum ; then
				log "checksum for directory $path does not match"
				exit 1
			fi
		fi
	done
	cd "$OLDPATH"

	PRODUCT=$(grep Product "$UPDATE_TMP/META-INF/MANIFEST.MF" | cut -d " " -f 2-)
	if [ "$PRODUCT" != "Display43" ] ; then
		log "update file is not a Display43 update file" $PRODUCT
		exit 1
	fi

	# delete ok directory
	[ -d "$UPDATE_OK" ] && rm -rf "$UPDATE_OK"

	mv "$UPDATE_TMP" "$UPDATE_OK"

	VERSION=$(grep Firmware "$UPDATE_OK/META-INF/MANIFEST.MF" | cut -d " " -f 2-)
	echo $VERSION >"$UPDATE_OK/version"

	log "successfully unzipped and checked update file"
	;;
update)
	if [ ! -d "$UPDATE_OK" ] ; then
		log "$UPDATE_OK does not exist"
		exit 1
	fi
	chmod 755 "$UPDATE_OK/sbin/update-rescue.sh"
	if ! "$UPDATE_OK/sbin/update-rescue.sh" "$UPDATE_OK" ; then
		log "error while installing rescue kernel"
		exit 1
	fi

	# delete final directory
	[ -d "$UPDATE_FINAL" ] && rm -rf "$UPDATE_FINAL"

	mv "$UPDATE_OK" "$UPDATE_FINAL"
	;;
*)
	echo "unknown command: " $2
	exit 1
	;;
esac

exit 0
